#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
Gestion de JdR
"""

import json
import tornado.ioloop
import tornado.web
import tornado.websocket

CLIENTS = []

class Plateaux():
    """Gestion des plateaux de jeu"""
    def __init__(self):
        self.plateaux = None
        self.plateau = None
        self.plateau_idx = None
        self.charger()

    def ajout_creature(self, nom, taille, couleur):
        """Ajout d'une creature"""
        creature = {
            "nom":nom,
            "coords":"0|0",
            "pv":0,
            "taille":taille,
            "couleur":couleur,
            "visible":False
        }
        self.plateau["creatures"].append(creature)

    def bascule_visibilite_carte(self, coords):
        """Bascule la visibilité du camouflage de coordonnées données"""
        if coords in self.plateau["visibles"]:
            self.plateau["visibles"].remove(coords)
        else:
            self.plateau["visibles"].append(coords)

    def bascule_visibilite_creature(self, nom_creature):
        """Bascule la visibilité d'un créature"""
        for creature in self.plateau["creatures"]:
            if creature["nom"] == nom_creature:
                creature["visible"] = not creature["visible"]

    def charger(self):
        """Charger les données JSON"""
        with open("plateaux.json") as fichier:
            self.plateaux = json.load(fichier)

    def choix(self, plateau_idx):
        """Choix du plateau de jeu"""
        if plateau_idx is None:
            self.plateau_idx = None
            self.plateau = None
        else:
            self.plateau_idx = plateau_idx
            self.plateau = self.plateaux[plateau_idx]

    def dplt_creature(self, nom_creature, destination):
        """Déplacement d'une créature"""
        for creature in self.plateau["creatures"]:
            if creature["nom"] == nom_creature:
                creature["coords"] = destination

    def enregistrer(self):
        """Enregistrement des données JSON"""
        if self.plateau_idx is not None:
            self.plateaux[self.plateau_idx] = self.plateau
        with open("plateaux.json", "w") as fichier:
            json.dump(self.plateaux,
                      fichier, sort_keys=True,
                      indent=4, ensure_ascii=False
                     )

    def liste(self):
        """Retourne la liste des plateaux de jeu"""
        liste = []
        for idx, plateau in enumerate(self.plateaux):
            liste.append((idx, plateau["nom"]))
        return liste

    def message(self, message):
        """Reception d'un message"""
        message = json.loads(message)
        mode = message["mode"]
        if mode == "basculevisibilitecarte":
            contenu = message["contenu"]
            self.bascule_visibilite_carte(contenu)
        elif mode == "dpltcreature":
            creature = message["creature"]
            destination = message["destination"]
            self.dplt_creature(creature, destination)
        elif mode == "modifpv":
            creature = message["creature"]
            pdv = message["pv"]
            self.modif_pv(creature, pdv)
        elif mode == "modifvisibilite":
            creature = message["creature"]
            self.bascule_visibilite_creature(creature)
        elif mode == "suppression":
            creature = message["creature"]
            self.suppression(creature)
        elif mode == "ajoutcreature":
            nom = message["nom"]
            taille = message["taille"]
            couleur = message["couleur"]
            self.ajout_creature(nom, taille, couleur)
        elif mode == "sauvegarde":
            self.enregistrer()

    def modif_pv(self, nom_creature, pdv):
        """Modification des points de vie d'une créature"""
        for creature in self.plateau["creatures"]:
            if creature["nom"] == nom_creature:
                creature["pv"] = pdv

    def suppression(self, nom_creature):
        """Suppression d'une créature"""
        for creature in self.plateau["creatures"]:
            if creature["nom"] == nom_creature:
                self.plateau["creatures"].remove(creature)

class PJHTML(tornado.web.RequestHandler):
    """Classe pour la page d'accueil"""
    def get(self):
        if PLATEAUX.plateau_idx is None:
            self.render("templates/attente.html")
        else:
            self.render("templates/cartepj.html")


class DMHTML(tornado.web.RequestHandler):
    """Classe pour la page d'accueil"""
    def get(self):
        plateau_idx = self.get_query_argument("plateau_idx",
                                              None
                                             )
        if plateau_idx is None:
            PLATEAUX.choix(None)
            self.render("templates/choix.html",
                        noms=PLATEAUX.liste()
                       )
        else:
            plateau_idx = int(plateau_idx)
            PLATEAUX.choix(plateau_idx)
            self.render("templates/cartedm.html")


class WebSocket(tornado.websocket.WebSocketHandler):
    """Classe pour le websocket"""
    def open(self):
        global CLIENTS
        CLIENTS.append(self)
        message = {
            "mode":"initialisation",
            "contenu":PLATEAUX.plateau
            }
        message = json.dumps(message)
        self.write_message(message)

    def on_message(self, message):
        global CLIENTS
        for client in CLIENTS:
            client.write_message(message)
        PLATEAUX.message(message)

    def on_close(self):
        global CLIENTS
        CLIENTS.remove(self)

PLATEAUX = Plateaux()

APP = tornado.web.Application([
    (r"/", PJHTML),
    (r"/dm", DMHTML),
    (r"/websocket", WebSocket),
    (r"/static/(.*)", tornado.web.StaticFileHandler, {"path": "./static"}),
])
APP.listen(5000)
tornado.ioloop.IOLoop.current().start()
